@@ -1,14 +1,11 @@
 use 5.008001;
 use strict;
 use warnings;
-
 package Email::MIME;
-{
-  $Email::MIME::VERSION = '1.925';
-}
-use Email::Simple 2.102; # crlf handling
-use parent qw(Email::Simple);
 # ABSTRACT: easy MIME message handling
+$Email::MIME::VERSION = '1.937';
+use Email::Simple 2.206; # header_raw
+use parent qw(Email::Simple);
 
 use Carp ();
 use Email::MessageID;
@@ -19,8 +16,100 @@ use Email::MIME::Encodings 1.314;
 use Email::MIME::Header;
 use Email::MIME::Modifier;
 use Encode 1.9801 ();
-use Scalar::Util qw(reftype);
+use Scalar::Util qw(reftype weaken);
 
+#pod =head1 SYNOPSIS
+#pod
+#pod B<Wait!>  Before you read this, maybe you just need L<Email::Stuffer>, which is
+#pod a much easier-to-use tool for building simple email messages that might have
+#pod attachments or both plain text and HTML.  If that doesn't do it for you, then
+#pod by all means keep reading.
+#pod
+#pod   use Email::MIME;
+#pod   my $parsed = Email::MIME->new($message);
+#pod
+#pod   my @parts = $parsed->parts; # These will be Email::MIME objects, too.
+#pod   my $decoded = $parsed->body;
+#pod   my $non_decoded = $parsed->body_raw;
+#pod
+#pod   my $content_type = $parsed->content_type;
+#pod
+#pod ...or...
+#pod
+#pod   use Email::MIME;
+#pod   use IO::All;
+#pod
+#pod   # multipart message
+#pod   my @parts = (
+#pod       Email::MIME->create(
+#pod           attributes => {
+#pod               filename     => "report.pdf",
+#pod               content_type => "application/pdf",
+#pod               encoding     => "quoted-printable",
+#pod               name         => "2004-financials.pdf",
+#pod           },
+#pod           body => io( "2004-financials.pdf" )->binary->all,
+#pod       ),
+#pod       Email::MIME->create(
+#pod           attributes => {
+#pod               content_type => "text/plain",
+#pod               disposition  => "attachment",
+#pod               charset      => "US-ASCII",
+#pod           },
+#pod           body_str => "Hello there!",
+#pod       ),
+#pod   );
+#pod
+#pod   my $email = Email::MIME->create(
+#pod       header_str => [ From => 'casey@geeknest.com' ],
+#pod       parts      => [ @parts ],
+#pod   );
+#pod
+#pod   # nesting parts
+#pod   $email->parts_set(
+#pod       [
+#pod         $email->parts,
+#pod         Email::MIME->create( parts => [ @parts ] ),
+#pod       ],
+#pod   );
+#pod
+#pod   # standard modifications
+#pod   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
+#pod   $email->header_str_set( To            => rcpts()        );
+#pod   $email->header_str_set( Cc            => aux_rcpts()    );
+#pod   $email->header_str_set( Bcc           => sekrit_rcpts() );
+#pod
+#pod   # more advanced
+#pod   $_->encoding_set( 'base64' ) for $email->parts;
+#pod
+#pod   # Quick multipart creation
+#pod   my $quicky = Email::MIME->create(
+#pod       header_str => [
+#pod           From => 'my@address',
+#pod           To   => 'your@address',
+#pod       ],
+#pod       parts => [
+#pod           q[This is part one],
+#pod           q[This is part two],
+#pod           q[These could be binary too],
+#pod       ],
+#pod   );
+#pod
+#pod   print $email->as_string;
+#pod
+#pod =head1 DESCRIPTION
+#pod
+#pod This is an extension of the L<Email::Simple> module, to handle MIME
+#pod encoded messages. It takes a message as a string, splits it up into its
+#pod constituent parts, and allows you access to various parts of the
+#pod message. Headers are decoded from MIME encoding.
+#pod
+#pod =head1 METHODS
+#pod
+#pod Please see L<Email::Simple> for the base set of methods. It won't take
+#pod very long. Added to that, you have:
+#pod
+#pod =cut
 
 our $CREATOR = 'Email::MIME::Creator';
 
@@ -36,6 +125,55 @@ sub new {
   return $self;
 }
 
+#pod =method create
+#pod
+#pod   my $single = Email::MIME->create(
+#pod     header_str => [ ... ],
+#pod     body_str   => '...',
+#pod     attributes => { ... },
+#pod   );
+#pod
+#pod   my $multi = Email::MIME->create(
+#pod     header_str => [ ... ],
+#pod     parts      => [ ... ],
+#pod     attributes => { ... },
+#pod   );
+#pod
+#pod This method creates a new MIME part. The C<header_str> parameter is a list of
+#pod headers pairs to include in the message. The value for each pair is expected to
+#pod be a text string that will be MIME-encoded as needed.  A similar C<header>
+#pod parameter can be provided in addition to or instead of C<header_str>.  Its
+#pod values will be used verbatim.
+#pod
+#pod C<attributes> is a hash of MIME attributes to assign to the part, and may
+#pod override portions of the header set in the C<header> parameter.
+#pod
+#pod The C<parts> parameter is a list reference containing C<Email::MIME>
+#pod objects. Elements of the C<parts> list can also be a non-reference
+#pod string of data. In that case, an C<Email::MIME> object will be created
+#pod for you. Simple checks will determine if the part is binary or not, and
+#pod all parts created in this fashion are encoded with C<base64>, just in case.
+#pod
+#pod If C<body> is given instead of C<parts>, it specifies the body to be used for a
+#pod flat (subpart-less) MIME message.  It is assumed to be a sequence of octets.
+#pod
+#pod If C<body_str> is given instead of C<body> or C<parts>, it is assumed to be a
+#pod character string to be used as the body.  If you provide a C<body_str>
+#pod parameter, you B<must> provide C<charset> and C<encoding> attributes.
+#pod
+#pod Back to C<attributes>. The hash keys correspond directly to methods or
+#pod modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
+#pod content_type, charset, name, format, boundary, encoding, disposition,
+#pod and filename. They will be mapped to C<"$attr\_set"> for message
+#pod modification.
+#pod
+#pod =cut
+
+my %CT_SETTER = map {; $_ => 1 } qw(
+  content_type charset name format boundary
+  encoding
+  disposition filename
+);
 
 sub create {
   my ($class, %args) = @_;
@@ -75,7 +213,7 @@ sub create {
   # parse_content_type(undef) which, for some reason, returns the default.
   # It's really sort of mind-boggling.  Anyway, the default ends up being
   # q{text/plain; charset="us-ascii"} so that if content_type is in the
-  # attributes, but not charset, then charset isn't changedand you up with
+  # attributes, but not charset, then charset isn't changed and you up with
   # something that's q{image/jpeg; charset="us-ascii"} and you look like a
   # moron. -- rjbs, 2009-01-20
   if (
@@ -86,13 +224,8 @@ sub create {
 
   my $email = $class->new($header);
 
-  foreach (qw(
-    content_type charset name format boundary
-    encoding
-    disposition filename
-  )) {
-    my $set = "$_\_set";
-    $email->$set($attrs{$_}) if exists $attrs{$_};
+  for my $key (keys %attrs) {
+    $email->content_type_attribute_set($key => $attrs{$key});
   }
 
   my $body_args = grep { defined $args{$_} } qw(parts body body_str);
@@ -235,7 +368,7 @@ sub parts_multipart {
   $self->SUPER::body_set(undef);
 
   # If there are no headers in the potential MIME part, it's just part of the
-  # body.  This is a horrible hack, although it's debateable whether it was
+  # body.  This is a horrible hack, although it's debatable whether it was
   # better or worse when it was $self->{body} = shift @bits ... -- rjbs,
   # 2006-11-27
   $self->SUPER::body_set(shift @bits) if ($bits[0] || '') !~ /.*:.*/;
@@ -245,7 +378,7 @@ sub parts_multipart {
   my @parts;
   for my $bit (@bits) {
     $bit =~ s/\A[\n\r]+//smg;
-    $bit =~ s/$self->{mycrlf}\Z//smg;
+    $bit =~ s/(?<!\x0d)$self->{mycrlf}\Z//sm;
     my $email = (ref $self)->new($bit);
     push @parts, $email;
   }
@@ -264,8 +397,8 @@ sub debug_structure {
   $level ||= 0;
   my $rv = " " x (5 * $level);
   $rv .= "+ " . ($self->content_type || '') . "\n";
-  my @parts = $self->parts;
-  if (@parts > 1) { $rv .= $_->debug_structure($level + 1) for @parts; }
+  my @parts = $self->subparts;
+  $rv .= $_->debug_structure($level + 1) for @parts;
   return $rv;
 }
 
@@ -300,11 +433,29 @@ sub invent_filename {
 
 sub default_header_class { 'Email::MIME::Header' }
 
+sub header_str {
+  my $self = shift;
+  $self->header_obj->header_str(@_);
+}
+
 sub header_str_set {
   my $self = shift;
   $self->header_obj->header_str_set(@_);
 }
 
+sub header_str_pairs {
+  my $self = shift;
+  $self->header_obj->header_str_pairs(@_);
+}
+
+#pod =method content_type_set
+#pod
+#pod   $email->content_type_set( 'text/html' );
+#pod
+#pod Change the content type. All C<Content-Type> header attributes
+#pod will remain intact.
+#pod
+#pod =cut
 
 sub content_type_set {
   my ($self, $ct) = @_;
@@ -315,6 +466,24 @@ sub content_type_set {
   return $ct;
 }
 
+#pod =method charset_set
+#pod
+#pod =method name_set
+#pod
+#pod =method format_set
+#pod
+#pod =method boundary_set
+#pod
+#pod   $email->charset_set( 'UTF-8' );
+#pod   $email->name_set( 'some_filename.txt' );
+#pod   $email->format_set( 'flowed' );
+#pod   $email->boundary_set( undef ); # remove the boundary
+#pod
+#pod These four methods modify common C<Content-Type> attributes. If set to
+#pod C<undef>, the attribute is removed. All other C<Content-Type> header
+#pod information is preserved when modifying an attribute.
+#pod
+#pod =cut
 
 BEGIN {
   foreach my $attr (qw[charset name format]) {
@@ -349,15 +518,62 @@ sub boundary_set {
   $self->parts_set([ $self->parts ]) if $self->parts > 1;
 }
 
+sub content_type_attribute_set {
+  my ($self, $key, $value) = @_;
+  $key = lc $key;
+
+  if ($CT_SETTER{$key}) {
+    my $method = "$key\_set";
+    return $self->$method($value);
+  }
+
+  my $ct_header = parse_content_type($self->header('Content-Type'));
+  my $attrs = $ct_header->{attributes};
+
+  for my $existing_key (keys %$attrs) {
+    delete $attrs->{$existing_key} if lc $existing_key eq $key;
+  }
+
+  if ($value) {
+    $ct_header->{attributes}->{$key} = $value;
+  } else {
+    delete $ct_header->{attributes}->{$key};
+  }
+  $self->_compose_content_type($ct_header);
+}
+
+#pod =method encoding_set
+#pod
+#pod   $email->encoding_set( 'base64' );
+#pod   $email->encoding_set( 'quoted-printable' );
+#pod   $email->encoding_set( '8bit' );
+#pod
+#pod Convert the message body and alter the C<Content-Transfer-Encoding>
+#pod header using this method. Your message body, the output of the C<body()>
+#pod method, will remain the same. The raw body, output with the C<body_raw()>
+#pod method, will be changed to reflect the new encoding.
+#pod
+#pod =cut
 
 sub encoding_set {
   my ($self, $enc) = @_;
   $enc ||= '7bit';
   my $body = $self->body;
-  $self->header_set('Content-Transfer-Encoding' => $enc);
+  $self->header_raw_set('Content-Transfer-Encoding' => $enc);
   $self->body_set($body);
 }
 
+#pod =method body_set
+#pod
+#pod   $email->body_set( $unencoded_body_string );
+#pod
+#pod This method will encode the new body you send using the encoding
+#pod specified in the C<Content-Transfer-Encoding> header, then set
+#pod the body to the new encoded body.
+#pod
+#pod This method overrides the default C<body_set()> method.
+#pod
+#pod =cut
 
 sub body_set {
   my ($self, $body) = @_;
@@ -367,7 +583,6 @@ sub body_set {
     Carp::croak("provided body reference is not a scalar reference")
       unless reftype($body) eq 'SCALAR';
     $body_ref = $body;
-    $body     = $$body_ref;
   } else {
     $body_ref = \$body;
   }
@@ -379,14 +594,27 @@ sub body_set {
   # Simple subclasses were free to alter the guts of the Email::Simple
   # object. -- rjbs, 2007-07-16
   unless (((caller(1))[3] || '') eq 'Email::Simple::new') {
-    $body = Email::MIME::Encodings::encode($enc, $body)
+    $$body_ref = Email::MIME::Encodings::encode($enc, $$body_ref)
       unless !$enc || $enc =~ $NO_ENCODE_RE;
   }
 
-  $self->{body_raw} = $body;
+  $self->{body_raw} = $$body_ref;
   $self->SUPER::body_set($body_ref);
 }
 
+#pod =method body_str_set
+#pod
+#pod   $email->body_str_set($unicode_str);
+#pod
+#pod This method behaves like C<body_set>, but assumes that the given value is a
+#pod Unicode string that should be encoded into the message's charset
+#pod before being set.
+#pod
+#pod The charset must already be set, either manually (via the C<attributes>
+#pod argument to C<create> or C<charset_set>) or through the C<Content-Type> of a
+#pod parsed message.  If the charset can't be determined, an exception is thrown.
+#pod
+#pod =cut
 
 sub body_str_set {
   my ($self, $body_str) = @_;
@@ -399,6 +627,14 @@ sub body_str_set {
   $self->body_set($body_octets);
 }
 
+#pod =method disposition_set
+#pod
+#pod   $email->disposition_set( 'attachment' );
+#pod
+#pod Alter the C<Content-Disposition> of a message. All header attributes
+#pod will remain intact.
+#pod
+#pod =cut
 
 sub disposition_set {
   my ($self, $dis) = @_;
@@ -407,9 +643,17 @@ sub disposition_set {
   $dis_header
     ? ($dis_header =~ s/^([^;]+)/$dis/)
     : ($dis_header = $dis);
-  $self->header_set('Content-Disposition' => $dis_header);
+  $self->header_raw_set('Content-Disposition' => $dis_header);
 }
 
+#pod =method filename_set
+#pod
+#pod   $email->filename_set( 'boo.pdf' );
+#pod
+#pod Sets the filename attribute in the C<Content-Disposition> header. All other
+#pod header information is preserved when setting this attribute.
+#pod
+#pod =cut
 
 sub filename_set {
   my ($self, $filename) = @_;
@@ -428,9 +672,19 @@ sub filename_set {
     $dis .= qq[; $attr="$val"];
   }
 
-  $self->header_set('Content-Disposition' => $dis);
+  $self->header_raw_set('Content-Disposition' => $dis);
 }
 
+#pod =method parts_set
+#pod
+#pod   $email->parts_set( \@new_parts );
+#pod
+#pod Replaces the parts for an object. Accepts a reference to a list of
+#pod C<Email::MIME> objects, representing the new parts. If this message was
+#pod originally a single part, the C<Content-Type> header will be changed to
+#pod C<multipart/mixed>, and given a new boundary attribute.
+#pod
+#pod =cut
 
 sub parts_set {
   my ($self, $parts) = @_;
@@ -450,11 +704,16 @@ sub parts_set {
     $body .= "$self->{mycrlf}--$bound--$self->{mycrlf}";
     @{$ct_header}{qw[type subtype]} = qw[multipart mixed]
       unless grep { $ct_header->{type} eq $_ } qw[multipart message];
+    $self->encoding_set('7bit');
+    delete $ct_header->{attributes}{charset};
   } elsif (@$parts == 1) {  # setup singlepart
     $body .= $parts->[0]->body;
-    @{$ct_header}{qw[type subtype]}
-      = @{ parse_content_type($parts->[0]->header('Content-Type')) }
-      {qw[type subtype]};
+
+    my $from_ct = parse_content_type($parts->[0]->header('Content-Type'));
+    @{$ct_header}{qw[type subtype]} = @{ $from_ct }{qw[type subtype]};
+
+    $ct_header->{attributes}{charset} = $from_ct->{attributes}{charset};
+
     $self->encoding_set($parts->[0]->header('Content-Transfer-Encoding'));
     delete $ct_header->{attributes}->{boundary};
   }
@@ -465,29 +724,66 @@ sub parts_set {
   $self->_reset_cids;
 }
 
+#pod =method parts_add
+#pod
+#pod   $email->parts_add( \@more_parts );
+#pod
+#pod Adds MIME parts onto the current MIME part. This is a simple extension
+#pod of C<parts_set> to make our lives easier. It accepts an array reference
+#pod of additional parts.
+#pod
+#pod =cut
 
 sub parts_add {
   my ($self, $parts) = @_;
   $self->parts_set([ $self->parts, @{$parts}, ]);
 }
 
+#pod =method walk_parts
+#pod
+#pod   $email->walk_parts(sub {
+#pod       my ($part) = @_;
+#pod       return if $part->subparts; # multipart
+#pod
+#pod       if ( $part->content_type =~ m[text/html]i ) {
+#pod           my $body = $part->body;
+#pod           $body =~ s/<link [^>]+>//; # simple filter example
+#pod           $part->body_set( $body );
+#pod       }
+#pod   });
+#pod
+#pod Walks through all the MIME parts in a message and applies a callback to
+#pod each. Accepts a code reference as its only argument. The code reference
+#pod will be passed a single argument, the current MIME part within the
+#pod top-level MIME object. All changes will be applied in place.
+#pod
+#pod =cut
 
 sub walk_parts {
   my ($self, $callback) = @_;
 
   my %changed;
 
-  my $walk;
-  $walk = sub {
+  my $walk_weak;
+  my $walk = sub {
     my ($part) = @_;
     $callback->($part);
 
     if (my @orig_subparts = $part->subparts) {
-      my @subparts = map {; $walk->($_) } @orig_subparts;
-      my $differ
-        =  (@subparts != @orig_subparts)
-        || (grep { $subparts[$_] != $orig_subparts[$_] } (0 .. $#subparts))
-        || (grep { $changed{ 0+$subparts[$_] } } (0 .. $#subparts));
+      my $differ;
+      my @subparts;
+
+      for my $part (@orig_subparts) {
+        my $str = $part->as_string;
+        next unless my $new = $walk_weak->($part);
+        $differ = 1 if $str ne $new->as_string;
+        push @subparts, $new;
+      }
+
+      $differ
+        ||=  (@subparts != @orig_subparts)
+        ||   (grep { $subparts[$_] != $orig_subparts[$_] } (0 .. $#subparts))
+        ||   (grep { $changed{ 0+$subparts[$_] } } (0 .. $#subparts));
 
       if ($differ) {
         $part->parts_set(\@subparts);
@@ -498,6 +794,9 @@ sub walk_parts {
     return $part;
   };
 
+  $walk_weak = $walk;
+  weaken $walk_weak;
+
   my $rv = $walk->($self);
 
   undef $walk;
@@ -509,9 +808,10 @@ sub _compose_content_type {
   my ($self, $ct_header) = @_;
   my $ct = join q{/}, @{$ct_header}{qw[type subtype]};
   for my $attr (sort keys %{ $ct_header->{attributes} }) {
-    $ct .= qq[; $attr="$ct_header->{attributes}{$attr}"];
+    next unless defined (my $value = $ct_header->{attributes}{$attr});
+    $ct .= qq[; $attr="$value"];
   }
-  $self->header_set('Content-Type' => $ct);
+  $self->header_raw_set('Content-Type' => $ct);
   $self->{ct} = $ct_header;
 }
 
@@ -537,11 +837,11 @@ sub _reset_cids {
       return if keys(%cids) == 1;
 
       my $cid = $self->_get_cid;
-      $_->header_set('Content-ID' => "<$cid>") for $self->parts;
+      $_->header_raw_set('Content-ID' => "<$cid>") for $self->parts;
     } else {
       foreach ($self->parts) {
         my $cid = $self->_get_cid;
-        $_->header_set('Content-ID' => "<$cid>")
+        $_->header_raw_set('Content-ID' => "<$cid>")
           unless $_->header('Content-ID');
       }
     }
@@ -550,17 +850,17 @@ sub _reset_cids {
 
 1;
 
-__END__
-
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.925
+version 1.937
 
 =head1 SYNOPSIS
 
@@ -580,7 +880,7 @@ by all means keep reading.
 
 ...or...
 
-  use Email::MIME::Creator;
+  use Email::MIME;
   use IO::All;
 
   # multipart message
@@ -592,7 +892,7 @@ by all means keep reading.
               encoding     => "quoted-printable",
               name         => "2004-financials.pdf",
           },
-          body => io( "2004-financials.pdf" )->all,
+          body => io( "2004-financials.pdf" )->binary->all,
       ),
       Email::MIME->create(
           attributes => {
@@ -616,7 +916,7 @@ by all means keep reading.
         Email::MIME->create( parts => [ @parts ] ),
       ],
   );
-  
+
   # standard modifications
   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
   $email->header_str_set( To            => rcpts()        );
@@ -625,7 +925,7 @@ by all means keep reading.
 
   # more advanced
   $_->encoding_set( 'base64' ) for $email->parts;
-  
+
   # Quick multipart creation
   my $quicky = Email::MIME->create(
       header_str => [
@@ -638,7 +938,7 @@ by all means keep reading.
           q[These could be binary too],
       ],
   );
-  
+
   print $email->as_string;
 
 =head1 DESCRIPTION
@@ -650,6 +950,9 @@ message. Headers are decoded from MIME encoding.
 
 =head1 METHODS
 
+Please see L<Email::Simple> for the base set of methods. It won't take
+very long. Added to that, you have:
+
 =head2 create
 
   my $single = Email::MIME->create(
@@ -785,7 +1088,7 @@ of additional parts.
   $email->walk_parts(sub {
       my ($part) = @_;
       return if $part->subparts; # multipart
-      
+
       if ( $part->content_type =~ m[text/html]i ) {
           my $body = $part->body;
           $body =~ s/<link [^>]+>//; # simple filter example
@@ -798,13 +1101,30 @@ each. Accepts a code reference as its only argument. The code reference
 will be passed a single argument, the current MIME part within the
 top-level MIME object. All changes will be applied in place.
 
+=head2 header
+
+B<Achtung!>  Beware this method!  In Email::MIME, it means the same as
+C<header_str>, but on an Email::Simple object, it means C<header_raw>.  Unless
+you always know what kind of object you have, you could get one of two
+significantly different behaviors.
+
+Try to use either C<header_str> or C<header_raw> as appropriate.
+
 =head2 header_str_set
 
   $email->header_str_set($header_name => @value_strings);
 
-This behaves like C<header_set>, but expects Unicode (character) strings as the
-values to set, rather than pre-encoded byte strings.  It will encode them as
-MIME encoded-words if they contain any control or 8-bit characters.
+This behaves like C<header_raw_set>, but expects Unicode (character) strings as
+the values to set, rather than pre-encoded byte strings.  It will encode them
+as MIME encoded-words if they contain any control or 8-bit characters.
+
+=head2 header_str_pairs
+
+  my @pairs = $email->header_str_pairs;
+
+This method behaves like C<header_raw_pairs>, returning a list of field
+name/value pairs, but the values have been decoded to character strings, when
+possible.
 
 =head2 parts
 
@@ -881,11 +1201,6 @@ For example:
     + text/plain
     + text/html
 
-=head1 METHODS
-
-Please see L<Email::Simple> for the base set of methods. It won't take
-very long. Added to that, you have:
-
 =head1 TODO
 
 All of the Email::MIME-specific guts should move to a single entry on the
@@ -919,6 +1234,66 @@ Simon Cozens <simon@cpan.org>
 
 =back
 
+=head1 CONTRIBUTORS
+
+=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Shawn Sorichetti Tomohiro Hosaka
+
+=over 4
+
+=item *
+
+Alex Vandiver <alexmv@mit.edu>
+
+=item *
+
+Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
+
+=item *
+
+Brian Cassidy <bricas@cpan.org>
+
+=item *
+
+David Steinbrunner <dsteinbrunner@pobox.com>
+
+=item *
+
+Dotan Dimet <dotan@corky.net>
+
+=item *
+
+Geraint Edwards <gedge-oss@yadn.org>
+
+=item *
+
+Jesse Luehrs <doy@tozt.net>
+
+=item *
+
+Kurt Anderson <kboth@drkurt.com>
+
+=item *
+
+Lance A. Brown <lance@bearcircle.net>
+
+=item *
+
+Matthew Horsfall (alh) <wolfsage@gmail.com>
+
+=item *
+
+memememomo <memememomo@gmail.com>
+
+=item *
+
+Shawn Sorichetti <ssoriche@coloredblocks.com>
+
+=item *
+
+Tomohiro Hosaka <bokutin@bokut.in>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2004 by Simon Cozens and Casey West.
@@ -927,3 +1302,122 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+__END__
+
+#pod =method header
+#pod
+#pod B<Achtung!>  Beware this method!  In Email::MIME, it means the same as
+#pod C<header_str>, but on an Email::Simple object, it means C<header_raw>.  Unless
+#pod you always know what kind of object you have, you could get one of two
+#pod significantly different behaviors.
+#pod
+#pod Try to use either C<header_str> or C<header_raw> as appropriate.
+#pod
+#pod =method header_str_set
+#pod
+#pod   $email->header_str_set($header_name => @value_strings);
+#pod
+#pod This behaves like C<header_raw_set>, but expects Unicode (character) strings as
+#pod the values to set, rather than pre-encoded byte strings.  It will encode them
+#pod as MIME encoded-words if they contain any control or 8-bit characters.
+#pod
+#pod =method header_str_pairs
+#pod
+#pod   my @pairs = $email->header_str_pairs;
+#pod
+#pod This method behaves like C<header_raw_pairs>, returning a list of field
+#pod name/value pairs, but the values have been decoded to character strings, when
+#pod possible.
+#pod
+#pod =method parts
+#pod
+#pod This returns a list of C<Email::MIME> objects reflecting the parts of the
+#pod message. If it's a single-part message, you get the original object back.
+#pod
+#pod In scalar context, this method returns the number of parts.
+#pod
+#pod This is a stupid method.  Don't use it.
+#pod
+#pod =method subparts
+#pod
+#pod This returns a list of C<Email::MIME> objects reflecting the parts of the
+#pod message.  If it's a single-part message, this method returns an empty list.
+#pod
+#pod In scalar context, this method returns the number of subparts.
+#pod
+#pod =method body
+#pod
+#pod This decodes and returns the body of the object I<as a byte string>. For
+#pod top-level objects in multi-part messages, this is highly likely to be something
+#pod like "This is a multi-part message in MIME format."
+#pod
+#pod =method body_str
+#pod
+#pod This decodes both the Content-Transfer-Encoding layer of the body (like the
+#pod C<body> method) as well as the charset encoding of the body (unlike the C<body>
+#pod method), returning a Unicode string.
+#pod
+#pod If the charset is known, it is used.  If there is no charset but the content
+#pod type is either C<text/plain> or C<text/html>, us-ascii is assumed.  Otherwise,
+#pod an exception is thrown.
+#pod
+#pod =method body_raw
+#pod
+#pod This returns the body of the object, but doesn't decode the transfer encoding.
+#pod
+#pod =method decode_hook
+#pod
+#pod This method is called before the L<Email::MIME::Encodings> C<decode> method, to
+#pod decode the body of non-binary messages (or binary messages, if the
+#pod C<force_decode_hook> method returns true).  By default, this method does
+#pod nothing, but subclasses may define behavior.
+#pod
+#pod This method could be used to implement the decryption of content in secure
+#pod email, for example.
+#pod
+#pod =method content_type
+#pod
+#pod This is a shortcut for access to the content type header.
+#pod
+#pod =method filename
+#pod
+#pod This provides the suggested filename for the attachment part. Normally
+#pod it will return the filename from the headers, but if C<filename> is
+#pod passed a true parameter, it will generate an appropriate "stable"
+#pod filename if one is not found in the MIME headers.
+#pod
+#pod =method invent_filename
+#pod
+#pod   my $filename = Email::MIME->invent_filename($content_type);
+#pod
+#pod This routine is used by C<filename> to generate filenames for attached files.
+#pod It will attempt to choose a reasonable extension, falling back to F<dat>.
+#pod
+#pod =method debug_structure
+#pod
+#pod   my $description = $email->debug_structure;
+#pod
+#pod This method returns a string that describes the structure of the MIME entity.
+#pod For example:
+#pod
+#pod   + multipart/alternative; boundary="=_NextPart_2"; charset="BIG-5"
+#pod     + text/plain
+#pod     + text/html
+#pod
+#pod =head1 TODO
+#pod
+#pod All of the Email::MIME-specific guts should move to a single entry on the
+#pod object's guts.  This will require changes to both Email::MIME and
+#pod L<Email::MIME::Modifier>, sadly.
+#pod
+#pod =head1 SEE ALSO
+#pod
+#pod L<Email::Simple>, L<Email::MIME::Modifier>, L<Email::MIME::Creator>.
+#pod
+#pod =head1 THANKS
+#pod
+#pod This module was generously sponsored by Best Practical
+#pod (http://www.bestpractical.com/), Pete Sergeant, and Pobox.com.
+#pod
+#pod =cut
